import exceptionalError from 'exceptionalhandler';
import util from 'util';
import * as shopConstants from './shopify.constants.js';
import * as logger from 'logger';
import * as shopAPI from './shopify.api.js';
import * as shopUtils from './shopify.utils.js';
// SHOPIFY ==> NICE ORDER
export function shopOrderToNiceOrder(shopOrder) {
    if (!shopOrder.hasOwnProperty("id")) {
        logger.info(util.inspect(shopOrder));
        throw new exceptionalError('No top level ID field - maybe you didnt provide the order object?');
    }
    let returnNiceOrder = setNiceOrderBaseValues(shopOrder);
    processShippingLines(returnNiceOrder, shopOrder.shipping_lines, shopOrder.id);
    processDiscountLines(returnNiceOrder, shopOrder.discount_applications);
    processLineItems(returnNiceOrder, shopOrder.line_items);
    return returnNiceOrder;
}
function setNiceOrderBaseValues(shopOrder) {
    let address;
    if (shopOrder.shipping_address) {
        address = shopOrder.shipping_address;
    }
    else if (shopOrder.customer && shopOrder.customer.default_address) {
        address = shopOrder.customer.default_address;
    }
    else {
        throw new exceptionalError(`No shipping or default address provided for order ${shopOrder.name}`);
    }
    let returnVal = {
        shopId: String(shopOrder.id),
        created: new Date(shopOrder.created_at),
        currency: idCurrency(shopOrder),
        orderNumber: shopOrder.name,
        notes: shopOrder.note,
        orderStatusPage: shopOrder.order_status_url,
        isTest: shopOrder.test,
        discountCodes: [],
        source: shopOrder.source_name,
        customerId: String(shopOrder.customer.id),
        address: {
            name: address.name,
            addressOne: address.address1,
            addressTwo: address.address2,
            city: address.city,
            province: address.province_code,
            country: address.country_code,
            postalCode: address.zip,
            phone: address.phone,
            email: shopOrder.email,
            firstName: address.first_name,
            lastName: address.last_name,
            company: address.company
        },
        items: [],
        requestedShipping: null,
        money: getShopOrderMoney(shopOrder)
    };
    if (Math.abs(returnVal.money.subTotal + returnVal.money.shipping + returnVal.money.tax - returnVal.money.discount - returnVal.money.total) > 0.05) {
        logger.info(util.inspect(shopOrder, { depth: null }));
        logger.info(util.inspect(returnVal, { depth: null }));
        throw new exceptionalError(`Money mismatch on order ${shopOrder.id} - See log for info`);
    }
    return returnVal;
}
function getShopOrderMoney(shopOrder) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    let subTotal = (_b = (_a = shopOrder === null || shopOrder === void 0 ? void 0 : shopOrder.total_line_items_price_set) === null || _a === void 0 ? void 0 : _a.presentment_money) === null || _b === void 0 ? void 0 : _b.amount;
    let tax = (_d = (_c = shopOrder === null || shopOrder === void 0 ? void 0 : shopOrder.total_tax_set) === null || _c === void 0 ? void 0 : _c.presentment_money) === null || _d === void 0 ? void 0 : _d.amount;
    let shipping = (_f = (_e = shopOrder === null || shopOrder === void 0 ? void 0 : shopOrder.total_shipping_price_set) === null || _e === void 0 ? void 0 : _e.presentment_money) === null || _f === void 0 ? void 0 : _f.amount;
    let discount = (_h = (_g = shopOrder === null || shopOrder === void 0 ? void 0 : shopOrder.total_discounts_set) === null || _g === void 0 ? void 0 : _g.presentment_money) === null || _h === void 0 ? void 0 : _h.amount;
    let total = (_k = (_j = shopOrder === null || shopOrder === void 0 ? void 0 : shopOrder.total_price_set) === null || _j === void 0 ? void 0 : _j.presentment_money) === null || _k === void 0 ? void 0 : _k.amount;
    return {
        subTotal: Number(subTotal),
        tax: Number(tax),
        shipping: Number(shipping),
        discount: Number(discount),
        total: Number(total)
    };
}
function idCurrency(shopOrder) {
    var _a, _b;
    let shopCurrency = shopOrder.currency;
    //@ts-ignore
    let totalCurrency = (_b = (_a = shopOrder === null || shopOrder === void 0 ? void 0 : shopOrder.total_price_set) === null || _a === void 0 ? void 0 : _a.presentment_money) === null || _b === void 0 ? void 0 : _b.currency_code;
    if (totalCurrency && totalCurrency !== shopCurrency) {
        return totalCurrency;
    }
    return shopCurrency;
}
function processShippingLines(returnNiceOrder, shipping_lines, shopId) {
    let shippingService = null;
    for (const shippingLine of shipping_lines) {
        if (!shippingService) {
            shippingService = shippingLine.code;
        }
        if (shippingService !== shippingLine.code) {
            logger.info(util.inspect(shippingLine));
            logger.info(`Selected: ${shippingService}`);
            throw new exceptionalError(`Shopify order ${shopId} had conflicting shipping services ${shippingService} & ${shippingLine.code}`);
        }
    }
    returnNiceOrder.requestedShipping = shippingService;
}
function processDiscountLines(returnNiceOrder, discount_codes) {
    for (const discount of discount_codes) {
        let newDiscount = {
            title: discount.title || '',
            type: discount.value_type,
            amount: discount.value
        };
        returnNiceOrder.discountCodes.push(newDiscount);
    }
}
function processLineItems(returnNiceOrder, lineItems) {
    var _a, _b;
    for (const requestItem of lineItems) {
        let newItem = {
            lineId: String(requestItem.id),
            qty: requestItem.quantity,
            unitPrice: Number((_b = (_a = requestItem === null || requestItem === void 0 ? void 0 : requestItem.price_set) === null || _a === void 0 ? void 0 : _a.presentment_money) === null || _b === void 0 ? void 0 : _b.amount),
            sku: requestItem.sku,
            title: requestItem.title,
            variantId: String(requestItem.variant_id),
            variantTitle: requestItem.variant_title
        };
        returnNiceOrder.items.push(newItem);
    }
}
// NICE ==> SHOPIFY ORDER
export function niceOrderToShopOrder(niceOrder, shippingService, notifyUser, tag) {
    //@ts-ignore
    //Gotta ignore because I'm adding in the extra types which is technically illegal
    validateNewShopOrder(niceOrder, shippingService, notifyUser);
    let newOrder = setShopOrderBaseValues(niceOrder);
    addShopOrderAddress(niceOrder, newOrder);
    addShopOrderLineItems(niceOrder, newOrder);
    addShopOrderLineMonies(niceOrder, newOrder);
    newOrder.send_receipt = notifyUser;
    newOrder.send_fulfillment_receipt = notifyUser;
    if (!shopUtils.isValEmpty(tag)) {
        newOrder.tags = tag;
    }
    newOrder.shipping_lines = [{
            custom: true,
            price: shopUtils.roundCashToString(niceOrder.money.shipping),
            title: shippingService
        }];
    return newOrder;
}
function setShopOrderBaseValues(niceOrder) {
    let newOrder = new shopAPI.shopify.rest.Order({ session: shopAPI.shopSession });
    newOrder.email = niceOrder.address.email;
    newOrder.fulfillment_status = shopConstants.fulfillmentOutboundStatusNew;
    newOrder.name = niceOrder.orderNumber;
    newOrder.currency = niceOrder.currency;
    newOrder.test = !!niceOrder.isTest;
    return newOrder;
}
function addShopOrderAddress(niceOrder, newOrder) {
    newOrder.shipping_address = {
        first_name: niceOrder.address.firstName,
        last_name: niceOrder.address.lastName,
        address1: niceOrder.address.addressOne,
        city: niceOrder.address.city,
        country_code: niceOrder.address.country,
        province_code: niceOrder.address.province,
        name: `${niceOrder.address.firstName} ${niceOrder.address.lastName}`,
        zip: niceOrder.address.postalCode
    };
    if (!shopUtils.isValEmpty(niceOrder.address.addressTwo)) {
        newOrder.shipping_address.address2 = niceOrder.address.addressTwo;
    }
    if (!shopUtils.isValEmpty(niceOrder.address.phone)) {
        newOrder.shipping_address.phone = niceOrder.address.phone;
    }
    if (!shopUtils.isValEmpty(niceOrder.address.company)) {
        newOrder.shipping_address.company = niceOrder.address.company;
    }
}
function addShopOrderLineItems(niceOrder, newOrder) {
    newOrder.line_items = [];
    for (const orderLine of niceOrder.items) {
        let newOrderLine = {
            quantity: orderLine.qty,
            price: shopUtils.roundCashToString(orderLine.unitPrice),
            variant_id: Number(orderLine.variantId),
            name: orderLine.variantTitle,
            title: orderLine.title
        };
        newOrder.line_items.push(newOrderLine);
    }
}
function addShopOrderLineMonies(niceOrder, newOrder) {
    newOrder.total_line_items_price = shopUtils.roundCashToString(niceOrder.money.subTotal);
    newOrder.total_tax = shopUtils.roundCashToString(niceOrder.money.tax);
    newOrder.total_shipping_price_set = {
        shop_money: {
            amount: shopUtils.roundCashToString(niceOrder.money.shipping),
            currency_code: niceOrder.currency
        },
        presentment_money: {
            amount: shopUtils.roundCashToString(niceOrder.money.shipping),
            currency_code: niceOrder.currency
        }
    };
    newOrder.total_price = shopUtils.roundCashToString(niceOrder.money.total);
    newOrder.total_discounts = shopUtils.roundCashToString(niceOrder.money.discount);
}
function validateNewShopOrder(niceOrder, shippingService, notifyUser) {
    if (shopUtils.isValEmpty(niceOrder.items)) {
        throw new exceptionalError(`No items provided`);
    }
    shopUtils.checkAllFields(shopConstants.mandatoryAddressFields, niceOrder.address, "Address");
    shopUtils.checkAllFields(shopConstants.mandatoryTopLevelFields, niceOrder, "Top Level");
    for (const lineItem of niceOrder.items) {
        if (lineItem.unitPrice === null) {
            lineItem.unitPrice = 0;
        }
        shopUtils.checkAllFields(shopConstants.mandatoryLineItemFields, lineItem, "Line Item");
    }
    if (niceOrder.address.country.length !== 2 || niceOrder.address.province.length !== 2) {
        throw new exceptionalError("Country and province codes must be two character ISO standard");
    }
    if (![shopConstants.shipSpeedExpress, shopConstants.shipSpeedStandard, shopConstants.shipSpeedPriority].includes(shippingService)) {
        throw new exceptionalError(`Invalid shipping service ${shippingService}`);
    }
    if (typeof notifyUser !== 'boolean') {
        throw new exceptionalError(`notifyUser must be boolean`);
    }
    validateNiceMonies(niceOrder);
}
function validateNiceMonies(niceOrder) {
    for (const moneyField of shopConstants.mandatoryMoney) {
        //@ts-ignore
        if (niceOrder.money[moneyField] === null || niceOrder.money[moneyField] === -0) {
            //@ts-ignore
            niceOrder.money[moneyField] = 0;
            //@ts-ignore
        }
        else if (typeof niceOrder.money[moneyField] !== "number") {
            throw new exceptionalError(`Money field ${moneyField} must be a number`);
        }
    }
    if (!shopConstants.validCurrencies.includes(niceOrder.currency)) {
        throw new exceptionalError(`Invalid currency ${niceOrder.currency}`);
    }
    if (niceOrder.money.discount < 0) {
        throw new exceptionalError("Discount cannot be negative");
    }
    let calcSubTotal = niceOrder.items.map((a) => (a.qty * a.unitPrice)).reduce((total, current) => total + current, 0);
    if (shopUtils.roundCashToString(calcSubTotal) !== shopUtils.roundCashToString(niceOrder.money.subTotal)) {
        throw new exceptionalError(`Subtotal mismatch - line items totaled ${calcSubTotal} but subTotal was ${niceOrder.money.subTotal}`);
    }
    let total = niceOrder.money.subTotal + niceOrder.money.tax + niceOrder.money.shipping - niceOrder.money.discount;
    if (shopUtils.roundCashToString(total) !== shopUtils.roundCashToString(niceOrder.money.total)) {
        throw new exceptionalError(`Grand Total Mismatch on ${niceOrder.orderNumber} (${niceOrder.money.total} != ${total})`);
    }
}
