var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as shopAPI from './shopify.api.js';
import * as shopUtils from './shopify.utils.js';
import exceptionalError from 'exceptionalhandler';
import * as shopConstants from './shopify.constants.js';
import * as shopConverter from './shopify.conversions.js';
import * as logger from 'logger';
import util from 'util';
// GET ORDERS
export function getOrders(lookupFilters) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!shopAPI.shopSession) {
            throw new exceptionalError("No session set");
        }
        if (shopUtils.isValEmpty(lookupFilters)) {
            throw new exceptionalError("Order type is required");
        }
        let statusLookup;
        switch (lookupFilters) {
            case "new":
                statusLookup = shopConstants.orderStatusOpen;
                break;
            default:
                throw new exceptionalError(`Invalid order type ${lookupFilters}`);
        }
        let orderLookup = yield shopAPI.shopify.rest.Order.all({
            session: shopAPI.shopSession,
            status: statusLookup
        });
        let returnOrders = [];
        for (const shopOrder of orderLookup.data) {
            //@ts-ignore breaking the law, breaking the law
            returnOrders.push(shopConverter.shopOrderToNiceOrder(shopOrder));
        }
        return returnOrders;
    });
}
export function getOrder(orderId) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!shopAPI.shopSession) {
            throw new exceptionalError("No session set");
        }
        if (shopUtils.isValEmpty(orderId)) {
            throw new exceptionalError("Order type is required");
        }
        let orderLookup = null;
        try {
            orderLookup = yield shopAPI.shopify.rest.Order.find({
                session: shopAPI.shopSession,
                id: orderId
            });
        }
        catch (e) {
            if (!e.message.match(/404/)) {
                logger.info(util.inspect(orderLookup, { depth: null }));
                throw new exceptionalError(`Order lookup for ${orderId} returned error ${e.message} - See log for more`);
            }
        }
        if (orderLookup) {
            return shopConverter.shopOrderToNiceOrder(orderLookup);
        }
        return orderLookup;
    });
}
// CHANGE ORDERS
export function shipOrder(orderId_1, trackingDetails_1) {
    return __awaiter(this, arguments, void 0, function* (orderId, trackingDetails, notifyUser = true) {
        if (!trackingDetails || Object.keys(trackingDetails).length === 0) {
            return;
        }
        let unfulfilledFulfillments = yield getUnshippedOrderFulfillmentIDs(orderId);
        let shipmentBundles = {};
        let processedVarIds = [];
        for (const [pkgId, trackingDeets] of Object.entries(trackingDetails)) {
            if (!trackingDeets.varIDs || trackingDeets.varIDs.length == 0) {
                throw new exceptionalError("Cant ship something without variant IDs");
            }
            for (const pkgVarId of trackingDeets.varIDs) {
                if (processedVarIds.includes(pkgVarId)) {
                    continue;
                }
                if (unfulfilledFulfillments.hasOwnProperty(pkgVarId)) {
                    const fulfillmentOrderId = unfulfilledFulfillments[pkgVarId].fulfillmentOrderId;
                    processedVarIds.push(pkgVarId);
                    if (!fulfillmentOrderId) {
                        logger.info(`Death on order ${orderId}`);
                        logger.info(util.inspect(trackingDetails, { depth: null }));
                        logger.info(util.inspect(unfulfilledFulfillments, { depth: null }));
                        throw new exceptionalError("No idea how we ended up here");
                    }
                    if (!unfulfilledFulfillments[pkgVarId].isShippable && (trackingDeets.trackingNumber || !trackingDeets.unshippable)) {
                        throw new exceptionalError("Can't have a non-shippable item with a carrier attached - do this separately");
                    }
                    if (!shipmentBundles.hasOwnProperty(pkgId)) {
                        shipmentBundles[pkgId] = {
                            tracking: trackingDeets.trackingNumber,
                            carrier: trackingDeets.carrier,
                            fulfillmentOrderIDs: [],
                            varIds: [],
                            isShippable: unfulfilledFulfillments[pkgVarId].isShippable
                        };
                    }
                    shipmentBundles[pkgId].fulfillmentOrderIDs.push(fulfillmentOrderId);
                    shipmentBundles[pkgId].varIds.push(pkgVarId);
                }
            }
        }
        for (const [pkgId, packageDeets] of Object.entries(shipmentBundles)) {
            yield shipPackage(packageDeets, notifyUser);
            for (const varId of packageDeets.varIds) {
                delete unfulfilledFulfillments[varId];
            }
        }
        if (Object.keys(unfulfilledFulfillments).length === 0) {
            yield closeOrder(orderId);
        }
    });
}
export function getUnshippedOrderFulfillmentIDs(orderId) {
    return __awaiter(this, void 0, void 0, function* () {
        let idMapping = {};
        let getOrderDeets = yield shopAPI.shopify.rest.FulfillmentOrder.all({
            session: shopAPI.shopSession,
            order_id: Number(orderId),
        });
        if (getOrderDeets.data.length === 0) {
            throw new exceptionalError(`No match for orderID ${orderId}`);
        }
        //@ts-ignore - more lazy shopify devs
        for (const fulfillmentOrder of getOrderDeets.data) {
            if (fulfillmentOrder.status !== shopConstants.orderStatusOpen) {
                continue;
            }
            for (const lineItem of fulfillmentOrder.line_items) {
                let varId = lineItem.variant_id;
                if (idMapping.hasOwnProperty(varId)) {
                    logger.info(util.inspect(getOrderDeets, { depth: null }));
                    throw new exceptionalError("Unexpected case where multiple line items have the same variant Id??");
                }
                let isShippable = true;
                if (fulfillmentOrder.delivery_method && fulfillmentOrder.delivery_method.method_type === 'none') {
                    isShippable = false;
                }
                idMapping[varId] = {
                    lineItemId: String(lineItem.line_item_id),
                    fulfillmentOrderId: String(lineItem.fulfillment_order_id),
                    qty: lineItem.quantity,
                    isShippable: isShippable
                };
            }
        }
        return idMapping;
    });
}
export function shipPackage(packageDeets_1) {
    return __awaiter(this, arguments, void 0, function* (packageDeets, notifyUser = true) {
        const fulfillmentUpdate = new shopAPI.shopify.rest.Fulfillment({ session: shopAPI.shopSession });
        if (!packageDeets.fulfillmentOrderIDs || !Array.isArray(packageDeets.fulfillmentOrderIDs) || shopUtils.isValEmpty(packageDeets.fulfillmentOrderIDs)) {
            throw new exceptionalError(`fulfillmentOrderIds must be an array with at least one fulfillment order ID`);
        }
        for (const fulfillmentOrderId of packageDeets.fulfillmentOrderIDs) {
            if (typeof fulfillmentOrderId !== 'string' || !fulfillmentOrderId.match(/^[0-9]+?$/)) {
                throw new exceptionalError(`Fulfillment order IDs must be numeric (but as a string.. yeah)`);
            }
        }
        if (packageDeets.isShippable && (shopUtils.isValEmpty(packageDeets.carrier) || shopUtils.isValEmpty(packageDeets.tracking))) {
            throw new exceptionalError(`Carrier and tracking number are mandatory fields`);
        }
        fulfillmentUpdate.line_items_by_fulfillment_order = [];
        for (const id of packageDeets.fulfillmentOrderIDs) {
            fulfillmentUpdate.line_items_by_fulfillment_order.push({ fulfillment_order_id: Number(id) });
        }
        fulfillmentUpdate.tracking_info = {
            number: packageDeets.tracking,
            company: packageDeets.carrier
        };
        // if(!shopUtils.isValEmpty(trackingDetails.url)){
        //     if(!trackingDetails.url!.match(/^http.+?\.\w{2,4}.*?$/i)){
        //       throw new exceptionalError(`Invalid tracking url`);
        //     }
        //     fulfillmentUpdate.tracking_info.url = trackingDetails.url;
        // }
        fulfillmentUpdate.notify_customer = notifyUser;
        let resp = yield shopAPI.submitObjectToAPI(fulfillmentUpdate);
    });
}
export function closeOrder(orderId) {
    return __awaiter(this, void 0, void 0, function* () {
        const order = new shopAPI.shopify.rest.Order({ session: shopAPI.shopSession });
        order.id = Number(orderId);
        yield order.close({});
    });
}
export function createOrder(niceNewOrder, shippingMethod) {
    return __awaiter(this, void 0, void 0, function* () {
        let newShopOrder = shopConverter.niceOrderToShopOrder(niceNewOrder, shippingMethod, shopConstants.sendEmails, shopConstants.tagFromSF);
        yield shopAPI.submitObjectToAPI(newShopOrder);
        return String(newShopOrder.id);
    });
}
